/*
 * Copyright (C) 2023  Valentin Lorentz
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License version 3,
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

use color_eyre::eyre::{Result, WrapErr};

use crate::App;

pub mod core;
pub mod memuse;

/// A [`Plugin`] that is not initialized yet
pub trait PrePlugin {
  fn name(&self) -> String;
  fn help(&self) -> String;
  fn load(self: Box<Self>, app: &mut App) -> Result<Box<dyn Plugin>>;
}

pub trait Plugin: PrePlugin {}

pub struct PluginGetter(pub fn() -> Result<Box<dyn PrePlugin>>);

inventory::collect!(PluginGetter);

pub fn load_all(app: &mut App) -> Result<()> {
  for plugin_getter in inventory::iter::<PluginGetter>() {
    // TODO: skip failing plugins instead of stopping load altogether
    let pre_plugin = (plugin_getter.0)().context("Could not get new plugin")?;
    let name = pre_plugin.name();
    pre_plugin
      .load(app)
      .with_context(|| format!("Could not load plugin {}", name))?;
  }

  Ok(())
}
